<?php

namespace HCM\FPA\Controllers;

use Zeedhi\Framework\Controller\Crud;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Request;
use HCM\Util\HCMParameters;
use HCM\FPA\Entity\FpatipomovimentEntity;
use HCM\FPA\Services\LancamentoEstruturaService;
use HCM\Util\Repositories;

class LancamentoEstruturaController extends Crud {

    protected $dataSourceName = 'getLancamentoEstrutura';
    protected $dataSourceName_Set = 'fpa_lancamentoestrutura_set';
    const TIPOESTRUT_TOMADOR = 7;

    protected $parameterBagColumns = array('P_NRORG', 'P_NRORG_PADRAO', 'P_NRCALCULOFOLHA', 'P_NRCALCULOFOLHA13', 'P_DTMESCOMPETENC');
    public function __construct (Manager $dataSourceManager, ParameterBag $parameterBag, HCMParameters $hcmParameters, LancamentoEstruturaService $LancamentoEstruturaService){
        $this->nrtipomoviment = FpatipomovimentEntity::TP_MOV_MENSAL;
        $this->nrtipomoviment13 = FpatipomovimentEntity::TP_MOV_13SAL;
        $this->parameterBag = $parameterBag;
        $this->hcmParameters = $hcmParameters;
        $this->LancamentoEstruturaService = $LancamentoEstruturaService;
        parent::__construct($dataSourceManager); 
    }

    public function getLancamentoEstrutura (DTO\Request\Filter $request, DTO\Response $response) {
        $filterData = $request->getFilterCriteria();
        $filterData->addCondition('P_NRCALCULOFOLHA','=',$this->hcmParameters->determinaNrCalculoFolha($this->nrtipomoviment));
        $filterData->addCondition('P_NRCALCULOFOLHA13','=',$this->hcmParameters->determinaNrCalculoFolha($this->nrtipomoviment13));
        $filterData->setDataSourceName($this->dataSourceName);
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData);
        $response->addDataSet($dataSet);
    }
    
    public function setLancamentoEstrutura(DTO\Request\DataSet $request, DTO\Response $response) {
        $this->dataSourceName = $this->dataSourceName_Set;
        try {
            foreach ($request->getDataSet()->getRows() as &$row) {
                $row['NRCALCULOFOLHA'] = $this->hcmParameters->determinaNrCalculoFolha($this->nrtipomoviment);

                if (array_key_exists('NRITECALCESTRUT', $row)) {
                    if ($row['NRITECALCESTRUT'] === '') {
                        $row['NRITECALCESTRUT'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'FPA_ITECALCESTRUT', 1, 12, true);
                    }
                } else {
                    $row['NRITECALCESTRUT'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'FPA_ITECALCESTRUT', 1, 12, true);
                }
                
            }
            parent::save($request, $response);
        } catch (\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function deleteLancamentoEstrutura(DTO\Request\DataSet $request, DTO\Response $response){
        try{
            $save = false;
            foreach ($request->getDataSet()->getRows() as &$row) {
                // Os eventos 3035 e 3036 permitem exclusão mas não exclusão física, apenas são desativados, pois são consequência do fechamento e reabertura do eSocial
                if ($row['NREVENTOM'] == 3035 || $row['NREVENTOM'] == 3036) {
                    $row['IDATIVO'] = 'N';
                    $save = true;
                }
            }
            if ($save) {
                $this->dataSourceName = $this->dataSourceName_Set;
                parent::save($request, $response);
            } else {
                parent::delete($request, $response);
            }
        } catch(\Exception $e){
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }    
    }
    
    public function cadAutoLancEstrutura(DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            $result = false;
            $nrorg = $this->hcmParameters->getNrorgTrabalho();
            $nrorgpadrao = $this->hcmParameters->getNrorgPadrao();
            $nrcalculofolha = $this->hcmParameters->determinaNrCalculoFolha($this->nrtipomoviment);
            $estrutGerencial = $this->hcmParameters->getTipoEstruturaGerencialOrg();
            
            foreach ($request->getDataSet()->getRows() as &$row) {
                $nrtipoestrutura = $row["NRTIPOESTRUTURA"];
                $arrTipoincide = $row["NRTIPOINCIDE"];
                $arrEstrutura = $row["NRESTRUTURAM"];
                $vrpercentincid = $row["VRPERCENTINCID"];
                $cdoperinclusao = $row["CDOPERINCLUSAO"];
                $cdoperultatu = $row["CDOPERULTATU"];
                $dtinclusao = $row["DTINCLUSAO"];
                $dtultatu = $row["DTULTATU"];
                $nrorginclusao = $row["NRORGINCLUSAO"];
                $nrorgultatu = $row["NRORGULTATU"];
                $nreventom = $row["NREVENTOM"];
                $dtocorrencia = $row["DTOCORRENCIA"];
                $this->LancamentoEstruturaService->entityManager->beginTransaction();
                
                foreach ($arrEstrutura as $nrestruturam) {
                    
                    $valorTotal = 0;
                    if ($nrtipoestrutura == $this->hcmParameters->getTipoEstruturaLegalOrg()) {
                        $valorTotal = $this->LancamentoEstruturaService->getValorTotalIncidencias($nrorg, $nrcalculofolha, $this->hcmParameters->getDtCompetencia(), $nrestruturam, 'A', $arrTipoincide);
                    }
    
                    if ($nrtipoestrutura == self::TIPOESTRUT_TOMADOR) {
                        $tipohierarq = $this->LancamentoEstruturaService->entityManager->getRepository(Repositories::TIPOHIERARQORG)->findBy(array('nrorg' => array($nrorg,$nrorgpadrao),
                        'nrtipoestrutsup' => $nrtipoestrutura,
                        'nrtipoestrutrel' => $estrutGerencial));
                        if (is_object($tipohierarq[0])) {
                            $estruturasRelac = $this->LancamentoEstruturaService->getEstruturasRelacionadas($nrorg, $tipohierarq[0], $nrestruturam);
                            if($estruturasRelac){
                                foreach ($estruturasRelac as $estrutRelac) {
                                    $valorTotal += $this->LancamentoEstruturaService->getValorTotalIncidencias($nrorg, $nrcalculofolha, $this->hcmParameters->getDtCompetencia(), $estrutRelac[0]->getNrestruturam(), 'A', $arrTipoincide);
                                }
                            } 
                        } else {
                            $this->LancamentoEstruturaService->entityManager->rollback();
                            throw new Exception("Não foi encontrado tipo de hierarquia 'Tomador/Estrutura Gerencial' para a organização");
                        }
                    }
                    if ($valorTotal > 0 && $vrpercentincid > 0) {
                        $valorTotal = $valorTotal * $vrpercentincid / 100;
                        $this->LancamentoEstruturaService->insertItem($nrorgultatu, $nrorginclusao, $dtultatu, $dtinclusao, $cdoperultatu, $cdoperinclusao, $nrorg, $nrcalculofolha, $nreventom, $dtocorrencia, $nrtipoestrutura, $nrestruturam, $valorTotal);
                        $result = true;
                    }    
                }
                
                $this->LancamentoEstruturaService->entityManager->commit();
            }
            if($result)
                $response->addNotification(new DTO\Response\Notification('cadAutoLancEstrutura'), DTO\Response\Notification::TYPE_SUCCESS);
            else
                $response->addNotification(new DTO\Response\Notification('HCM01077_FPA_VALOR_NULO', DTO\Response\Notification::TYPE_ERROR));
                
        } catch (\Exception $e) {
            $this->LancamentoEstruturaService->entityManager->rollback();
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    
     protected function processConditionValue($condition) {
        return $condition['value'];
    }
    protected function populateParameterBag(FilterCriteria $filterCriteria) {

        $conditions = array();
        foreach ($filterCriteria->getConditions() as $condition) {
            $columnName = $condition['columnName'];
            if (in_array($columnName, $this->parameterBagColumns)) {
                $this->parameterBag->set($columnName, $this->processConditionValue($condition));
            } else {
                $conditions[] = $condition;
            }
        }
        $newFilterCriteria = new FilterCriteria(
            $filterCriteria->getDataSourceName(),
            $conditions,
            $filterCriteria->getPage(),
            $filterCriteria->getPageSize()
        );


        return $newFilterCriteria;
    }
    
} 