<?php

namespace HCM\FPA\Controllers;

use Zeedhi\Framework\Controller\Crud;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DTO\Response\Notification;
use Zeedhi\Framework\DTO\Response\Message;
use HCM\Util\HCMParameters;
use Doctrine\DBAL\Connection;
use Doctrine\ORM\EntityManager;
use HCM\Util\DateUtil;
use HCM\Util\Repositories;
use HCM\Geral\Repository\RateioLancamentoRepository;

class RateioLancamentoController extends Crud {
    
    protected $rateioLancamentoService;
    protected $parameterBagColumns = array('P_NRORG', 'P_NRORG_PADRAO', 'P_DTMESCOMPETENC');
    public function __construct (Manager $dataSourceManager, ParameterBag $parameterBag, HCMParameters $hcmParameters, Connection $connection, EntityManager $entityManager, \HCM\FPA\Services\RateioLancamentoService $rateioLancamentoService, RateioLancamentoRepository $rateioLancamentoRepository)
    {
        $this->parameterBag = $parameterBag;
        $this->hcmParameters = $hcmParameters;
        $this->connection = $connection;
        $this->entityManager = $entityManager;
        $this->rateioLancamentoService = $rateioLancamentoService;
        $this->rateioLancamentoRepository = $rateioLancamentoRepository;
        parent::__construct($dataSourceManager);
    }

    public function getRateio(DTO\Request\Filter $request, DTO\Response $response)
    {   
        $this->dataSourceName = "fpa_rateio_get";
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName($this->dataSourceName);
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData);
        $response->addDataSet($dataSet);
    }

    public function setRateio(DTO\Request\DataSet $request, DTO\Response $response)
    {
        try {
            $this->dataSourceName = "fpa_rateio";
            foreach ($request->getDataSet()->getRows() as &$row) {
                $existeRateio = $this->rateioLancamentoService->buscaRateioExistente($row['NRORG'], $row['DTINIVIGENCIA'], $row['NRESTRUTURAM'], $row['NREVENTOM']);
                
                if($existeRateio && ($existeRateio['NRRATEIO'] != $row['NRRATEIO'] || !isset($row['NRRATEIO'])) ){
                    $response->addNotification(new Notification("Já existe um registro pra essa estrutura e evento com a mesma data de início vigência", Notification::TYPE_ALERT));
                    return;
                } 

                if($row['__is_new'] === true){
                    $row['NRRATEIO'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'FPA_RATEIO', 1, 12, true); 
                }
                
                parent::save($request, $response);
            }
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function deleteRateio(DTO\Request\DataSet $request, DTO\Response $response)
    {
        try{
            $this->dataSourceName = "fpa_rateio";
            $nrorg = $request->getDataSet()->getRows()[0]['NRORG'];
            $nrrateio = $request->getDataSet()->getRows()[0]['NRRATEIO'];

            $existeItens = $this->rateioLancamentoService->existeItensRateio($nrorg, $nrrateio);

            if(!$existeItens){
                parent::delete($request, $response);
            } else {
                $response->addNotification(new Notification("Existem itens relacionados a este registro, favor excluí-los.", Notification::TYPE_ALERT));
            }
        } catch(\Exception $e){
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }    
    }

    public function getRateioLancamento(DTO\Request\Filter $request, DTO\Response $response)
    {   
        $this->dataSourceName = "fpa_rateiolanc_get";
        $this->parameterBagColumns = array('P_NRORG', 'P_NRORG_PADRAO', 'P_NRRATEIO', 'P_DTMESCOMPETENC');

        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName($this->dataSourceName);
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData);
        $response->addDataSet($dataSet);
    }

    public function setRateioLancamento(DTO\Request\DataSet $request, DTO\Response $response)
    {   
        try {
            $this->dataSourceName = "fpa_rateiolanc";
            foreach ($request->getDataSet()->getRows() as &$row) {
                $row['VRFINAL'] = isset($row['VRFINAL']) ? $row['VRFINAL'] : NULL;
                $existeRateioLancamento = $this->rateioLancamentoService->buscaRateioExistenteLancamento($row['NRORG'], $row['NRRATEIO'], $row['DTLANCAMENTO']);
      
                if($existeRateioLancamento && ($existeRateioLancamento['NRRATEIOLANC'] != $row['NRRATEIOLANC'] || !isset($row['NRRATEIOLANC'])) ){
                    $response->addNotification(new Notification("Já existe um registro com essa data de lançamento", Notification::TYPE_ALERT));
                    return;
                }
             
                if($row['__is_new'] === true){
                    $row['NRRATEIOLANC'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'FPA_RATEIOLANC', 1, 12, true); 
                }
                parent::save($request, $response);
            }
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function deleteRateioLancamento(DTO\Request\DataSet $request, DTO\Response $response)
    {
        try{
            $this->dataSourceName = "fpa_rateiolanc";
            parent::delete($request, $response);
        } catch(\Exception $e){
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }  
    }

    public function getRateioEstrutura(DTO\Request\Filter $request, DTO\Response $response)
    {   
        $this->dataSourceName = "fpa_rateioestrut_get";
        $this->parameterBagColumns = array('P_NRORG', 'P_NRORG_PADRAO', 'P_NRRATEIO');
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName($this->dataSourceName);
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData);
        $response->addDataSet($dataSet);
    }

    public function setRateioEstrutura(DTO\Request\DataSet $request, DTO\Response $response)
    {
        try {
            $this->dataSourceName = "fpa_rateioestrut";
            foreach ($request->getDataSet()->getRows() as &$row) {
                $existeRateioEstrutura = $this->rateioLancamentoService->buscaRateioExistenteEstrutura($row['NRORG'], $row['NRESTRUTURAM'], $row[ 'NRRATEIO']);
                
                if($existeRateioEstrutura &&  ($existeRateioEstrutura['NRRATEIOESTRUT'] != $row['NRRATEIOESTRUT'] || !isset($row['NRRATEIOESTRUT'])) ){
                    $response->addNotification(new Notification("Já existe um registro com essa estrutura", Notification::TYPE_ALERT));
                    return;
                }  
                
                if($row['__is_new'] === true){
                    $row['NRRATEIOESTRUT'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'FPA_RATEIOESTRUT', 1, 12, true); 
                }
                parent::save($request, $response);
                $this->rateioLancamentoService->calculaPorcentagem($row['NRORG'],$row['NRRATEIO'], 'FPA_RATEIOESTRUT');
            }
            
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function deleteRateioEstrutura(DTO\Request\DataSet $request, DTO\Response $response)
    {
        try{
            $this->dataSourceName = "fpa_rateioestrut";
            parent::delete($request, $response);
        } catch(\Exception $e){
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }    
    }

    public function deleteRateioEstruturaChecked(DTO\Request\DataSet $request, DTO\Response $response){
        try{
            $this->dataSourceName = 'fpa_rateioestrut';
            parent::delete($request, $response);
        } catch(\Exception $e){
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }    
    }

    public function getRateioOcupacao(DTO\Request\Filter $request, DTO\Response $response)
    {   
        $this->dataSourceName = "fpa_rateioocup_get";
        $this->parameterBagColumns = array('P_NRORG', 'P_NRORG_PADRAO', 'P_DTMESCOMPETENC','P_NRRATEIO');
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName($this->dataSourceName);
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData);
        $response->addDataSet($dataSet);
    }

    public function setRateioOcupacao(DTO\Request\DataSet $request, DTO\Response $response)
    {
        try {
            $this->dataSourceName = "fpa_rateioocup";
            foreach ($request->getDataSet()->getRows() as &$row) {
                $row['VRFINAL'] = isset($row['VRFINAL']) ? $row['VRFINAL'] : NULL;
                $existeRateioOcupacao = $this->rateioLancamentoService->buscaRateioExistenteOcupacao($row['NRORG'],  $row['NRESTRUTURAM'], $row['NROCUPACAOM'], $row['NRRATEIO']);
                
                if($existeRateioOcupacao && ($existeRateioOcupacao['NRRATEIOOCUP'] != $row['NRRATEIOOCUP'] || !isset($row['NRRATEIOOCUP'])) ) {
                    $response->addNotification(new Notification("Já existe um registro pra essa ocupação e estrutura gerencial", Notification::TYPE_ALERT));
                    return;
                }  
               
                if($row['__is_new'] === true){
                    $row['NRRATEIOOCUP'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'FPA_RATEIOOCUP', 1, 12, true); 
                }

                parent::save($request, $response);
                $this->rateioLancamentoService->calculaPorcentagem($row['NRORG'],$row['NRRATEIO'], 'FPA_RATEIOOCUP');
            }
            
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }       
    }

    public function deleteRateioOcupacao(DTO\Request\DataSet $request, DTO\Response $response)
    {
        try{
            $this->dataSourceName = "fpa_rateioocup";
            parent::delete($request, $response);
        } catch(\Exception $e){
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }      
    }

    public function deleteRateioValores(DTO\Request\DataSet $request, DTO\Response $response)
    {
        try{
            $this->dataSourceName = "fpa_rateioval";
            parent::delete($request, $response);
        } catch(\Exception $e){
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }      
    }

    public function getRateioValores(DTO\Request\Filter $request, DTO\Response $response)
    {   
        $this->dataSourceName = "fpa_rateioval_get";
        $this->parameterBagColumns = array('P_NRORG', 'P_NRORG_PADRAO', 'P_NRRATEIO', 'P_DTMESCOMPETENC');
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName($this->dataSourceName);
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData);
        $response->addDataSet($dataSet);
    }


    public function setRateioValores(DTO\Request\DataSet $request, DTO\Response $response)
    {
        try {
            $this->dataSourceName = "fpa_rateioval";
            foreach ($request->getDataSet()->getRows() as &$row) {
                $dtmescompetenc = $this->hcmParameters->getDtCompetencia();
                $alert = false;
                $estrutLegal = $this->entityManager->getRepository(Repositories::FPA_RATEIO)->findOneBy(array('nrrateio' => $row['NRRATEIO'], 'nrorg' => $row['NRORG']));       
                $vinculos = $this->rateioLancamentoRepository->getVinculosEstrutLegal($row['NRORG'], $dtmescompetenc, $estrutLegal->getNrestruturam(), $row['NRVINCULOM_LIST']);
                if(count($vinculos) < 1) {
                    $response->addNotification(new Notification("Filtre por vínculos da estrutura legal do rateio", Notification::TYPE_ALERT));
                    return;
                }
                foreach ($vinculos as $vinculo) {
                    $row['NRVINCULOM'] = $vinculo->getNrvinculom(); 
                    $lancamentos = ((strlen($row['DTINICIO']) > 1) ? self::buscaLancamentosFiltro($row) : self::buscaLancamentosAtualCompetencia($row)); 
                   foreach($lancamentos as $lancamento) {
                    $estruturaGerenRateioVinculo = $this->entityManager->getRepository(Repositories::FPA_RATEIOESTRUT)->findOneBy(array('nrrateio' => $row['NRRATEIO'], 'nrorg' => $row['NRORG'], 'nrestruturam' => $vinculo->historico->getNrestrutgeren()));
                    $ocupacaoRateioVinculo = $this->entityManager->getRepository(Repositories::FPA_RATEIOOCUP)->findOneBy(array('nrrateio' => $row['NRRATEIO'], 'nrorg' => $row['NRORG'], 'nrestruturam' => $vinculo->historico->getNrestrutgeren(), 'nrocupacaom' => $vinculo->historico->getNrcargo()));
                        
                    if (!empty($estruturaGerenRateioVinculo) && !empty($ocupacaoRateioVinculo )) {
                            $row['NRVALOR']  = self::calculaRateio($lancamento->getVrfinal(), $estruturaGerenRateioVinculo->getVrporcentagem(), $ocupacaoRateioVinculo->getVrporcentagem());
                            $existeCalculoRateioLancamento = $this->rateioLancamentoService->buscaCalculoRateioExistente($row['NRORG'], $row['NRRATEIO'], $row['NRVINCULOM'], (DateUtil::getDataDeString($lancamento->getDtlancamento(),DateUtil::FORMATO_BRASILEIRO,true))->format('d/m/Y'));
                            if($existeCalculoRateioLancamento) {
                                $alert = true;
                                continue;
                            }
   
                            $row['DTDIALANC'] =(DateUtil::getDataDeString($lancamento->getDtlancamento(),DateUtil::FORMATO_BRASILEIRO,true))->format('d/m/Y');
                            $row['NRRATEIOVAL'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'FPA_RATEIOVAL', 1, 12, true); 
                            parent::save($request, $response);       
                        }       
                    }
                }

                if($alert) {
                    $response->addNotification(new Notification("Alguns rateios podem não ter sido gerado por já existirem", Notification::TYPE_ALERT));
                    return;
                }
            }
            
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }       
    }

    private function calculaRateio($vrfinal, $porcentagemEstrutGeren, $porcentagemOcupacao)
    {
        try {
            $resultado = ($vrfinal * ($porcentagemEstrutGeren/100)) * ($porcentagemOcupacao/100);
            return $resultado;
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }  
    }

    private function buscaLancamentosFiltro($row)
    {
        try {
            $dtmescompetenc = $this->hcmParameters->getDtCompetencia();
            $lancamentosRateio = $this->entityManager->getRepository(Repositories::FPA_RATEIOLANC)->findBy(array('nrrateio' => $row['NRRATEIO'], 'nrorg' => $row['NRORG']));
            $lancamentosRateioFiltro = array();
            foreach($lancamentosRateio as $lancamento) {
                $dtLancamento  = (DateUtil::getDataDeString($lancamento->getDtlancamento(),DateUtil::FORMATO_BRASILEIRO,true))->format('d/m/Y');

                if((strlen($row['DTINICIO']) > 1) && (strlen($row['DTFIM']) > 1)) {
                    if ($dtLancamento > $row['DTINICIO'] && $dtLancamento < $row['DTFIM']) {
                        array_push($lancamentosRateioFiltro, $lancamento);
                    }
                }

                if((strlen($row['DTINICIO']) > 1) && (strlen($row['DTFIM']) < 1)) {
                    if ($dtLancamento > $row['DTINICIO'] && $dtLancamento < $this->hcmParameters->getUltimoDiaMes()) {
                        array_push($lancamentosRateioFiltro, $lancamento);
                    }
                }
            }

            return $lancamentosRateioFiltro;
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }  
    }

    private function buscaLancamentosAtualCompetencia($row)
    {   
        try {
            $dtmescompetenc = $this->hcmParameters->getDtCompetencia();
            $lancamentosRateio = $this->entityManager->getRepository(Repositories::FPA_RATEIOLANC)->findBy(array('nrrateio' => $row['NRRATEIO'], 'nrorg' => $row['NRORG']));
            $lancamentosRateioCompAtual = array();
            foreach($lancamentosRateio as $lancamento) {
                $dtLancamento  = (DateUtil::getDataDeString($lancamento->getDtlancamento(),DateUtil::FORMATO_BRASILEIRO,true))->format('d/m/Y');
                if (substr($dtLancamento, 3,9) == substr($dtmescompetenc, 3,9) ) {
                    array_push($lancamentosRateioCompAtual, $lancamento);
                }
            }

            return $lancamentosRateioCompAtual;
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }  
        
    }


    public function getAuditoria(){
        $auditoria = array();
        $auditoria['DTINCLUSAO'] = DateUtil::getDataAtual()->format('d/m/Y');
        $auditoria['DTULTATU'] = DateUtil::getDataAtual()->format('d/m/Y');
        $auditoria['NRORGINCLUSAO'] = $this->hcmParameters->getNrorgTrabalho();
        $auditoria['CDOPERINCLUSAO'] = $this->hcmParameters->getCdoperador();
        $auditoria['NRORGULTATU'] = $this->hcmParameters->getNrorgTrabalho();
        $auditoria['CDOPERULTATU'] = $this->hcmParameters->getCdoperador();
        return $auditoria;
    }

    protected function processConditionValue($condition) {
        return $condition['value'];
    }

    protected function populateParameterBag(FilterCriteria $filterCriteria) {

        $conditions = array();
        foreach ($filterCriteria->getConditions() as $condition) {
            $columnName = $condition['columnName'];
            if (in_array($columnName, $this->parameterBagColumns)) {
                $this->parameterBag->set($columnName, $this->processConditionValue($condition));
            } else {
                $conditions[] = $condition;
            }
        }
        $newFilterCriteria = new FilterCriteria(
            $filterCriteria->getDataSourceName(),
            $conditions,
            $filterCriteria->getPage(),
            $filterCriteria->getPageSize()
        );
        return $newFilterCriteria;
    }
    
} 
