<?php

namespace HCM\FPA\ObrigacaoLegal\GRRF;

use Doctrine\ORM\EntityManager;
use HCM\Util\HCMParameters;
use HCM\Util\Messages;
use HCM\Util\Repositories;
use HCM\Util\DateUtil;
use HCM\Util\FPA;
use HCM\Util\TkDI;
use HCM\Util\MsgCalculo;
use HCM\Geral\Entity\GpevinculomEntity;
use HCM\Geral\Entity\GpepessoaEntity;
use HCM\Geral\Entity\GpesitufuncmEntity;
use HCM\Geral\Entity\GpevinculoempregEntity;
use HCM\Geral\Entity\GpeOcupacaomEntity;
use HCM\Geral\Entity\GpeCboEntity;
use HCM\Geral\Entity\GpealtesitufuncEntity;
use HCM\Geral\Entity\GpealtesalarioEntity; 
use HCM\Geral\Entity\GpetpmovtransfmEntity;
use HCM\Geral\Entity\GpemovimentacaoEntity;
use HCM\Geral\Entity\GpeescalatrabmEntity;
use HCM\Geral\Entity\EstruturamEntity;
use HCM\FPA\Entity\FpadepvinculoEntity;
use HCM\FPA\Entity\FpaTpdemissaoEntity;
use HCM\FPA\Entity\FpaitecalcfolhaEntity;
use HCM\FPA\Entity\FpaassoceventoEntity;
use HCM\FPA\Entity\FpaeventomEntity;
use HCM\FPA\Entity\FpagfipmagnetEntity;
use HCM\FPA\Entity\FpaValorreferenciamEntity;
use HCM\Geral\Entity\HierarquiaorgEntity;
use HCM\FPA\Calculos\Geral\Estruturacao;
use HCM\Geral\Entity\ContcorrparcEntity;
use HCM\FPA\Registro\grrf\Registro10GRRF;
use HCM\FPA\Registro\grrf\Registro40GRRF;
use HCM\FPA\Registro\grrf\Registro43GRRF;
use HCM\FPA\Registro\grrf\Registro44GRRF;
use HCM\FPA\Registro\grrf\Registro45GRRF;
use HCM\FPA\Registro\RegistroGRRF;

class GRRF{

    const ALFANUMERICOS         = '/[^a-zA-Z0-9]/';
    const ESTRUTURALEGAL        = 'ESTRUTURALEGAL';
    const ESTRUTURAGERENCIAL    = 'ESTRUTURAGERENCIAL';
    const NREVENTOLIQPARCIAL    = 3040;
    const NREVENTOLIQDEFINITIVO = 3050;

    private $_formulario;
    private $_nrOrg;
    private $_nrOrgPadrao;
    private $_cdOperador;
    private $_mesCompetencia;
    private $_dtInicio;
    private $_dtFim;
    private $_dtAux;
    private $_movimentacao;
    private $_estruturacao;
    private $_tpMovTransfM;
    private $_hierarquiaOrg;
    private $_registro;
    private $_registro10;
    private $_registro40;
    private $_registro43;
    private $_registro44;
    private $_registro45;
    private $_grrfMagnetico;
    private $_gfipMagneticoTrab;
    private $_dadoAlterado = array(array());
    private $_dadoVinculo = array();
    private $_dadoMesAnteriorRescisao = array();
    private $_dadoMesRescisao = array();
    private $_dadoAvisoPrevioIndenizado = array();
    private $_dadoSaldoFinsRescisorios = array();
    private $_dadoPensaoJuridica = array();
    private $_nrInscricao;
    private $_nrInscricaoTomador;
    private $_nrTpInscricao;
    private $_nrTpInscricaoTomador;
    private $_codMovimentacao;
    private $_codSaque;
    private $_pessoa;
    private $_sitFuncional;
    private $_vinculoEmpreg;
    private $_ocupacao;
    private $_cbo;
    private $_altSitFuncional;
    private $_alteracaoSalario;
    private $_tpDemissao;
    private $_itemCalculo;
    private $_assocEvento;
    private $_escalaTrabalho;
    private $_evento;
    private $_itemGrupo;
    private $_listaCalculoFolha;
    private $_listaCalculoFolhaAnt;
    private $_contaCorrenteParceiro;
    private $_dependentes;
    private $_valorReferencia;
    private $_infCompl43;
    private $_infCompl44;
    private $_infCompl45;
    private $_infComplComps;

    public function __construct($nrOrg, $nrOrgPadrao, $cdOperador, $mesCompetencia, $grrfMagnetico,
                                $formulario, $competencia13, $itensGrupo, $listaNrCalculoFolha, $entityManager, $hcmParameters, $listaNrCalculoFolhaAnt){

        $this->entityManager = $entityManager;
        $this->hcmParameters = $hcmParameters;
        $this->_nrOrg = $nrOrg;
        $this->_nrOrgPadrao = $nrOrgPadrao;
        $this->_cdOperador = $cdOperador;
        $this->_mesCompetencia = $mesCompetencia;
        $this->_dtInicio = $this->_mesCompetencia;
        $this->_dtFim = DateUtil::getUltimoDiaDoMes($this->_mesCompetencia);
        $this->_pessoa = new GpepessoaEntity($entityManager, $hcmParameters);
        $this->_sitFuncional = new GpesitufuncmEntity($entityManager, $hcmParameters);
        $this->_vinculoEmpreg = new GpevinculoempregEntity($entityManager, $hcmParameters);
        $this->_dependentes = new FpadepvinculoEntity($entityManager, $hcmParameters);
        $this->_ocupacao = new GpeOcupacaomEntity($entityManager, $hcmParameters);
        $this->_cbo = new GpeCboEntity($entityManager, $hcmParameters);
        $this->_altSitFuncional = new GpealtesitufuncEntity($entityManager, $hcmParameters);
        $this->_alteracaoSalario = new GpealtesalarioEntity($entityManager, $hcmParameters);
        $this->_tpDemissao = new FpaTpdemissaoEntity($entityManager, $hcmParameters);
        $this->_tpMovTransfM = new GpetpmovtransfmEntity($entityManager, $hcmParameters);
        $this->_listaCalculoFolha = $listaNrCalculoFolha;
        $this->_listaCalculoFolhaAnt = $listaNrCalculoFolhaAnt;
        $this->_itemCalculo = new FpaitecalcfolhaEntity($entityManager, $hcmParameters);
        $this->_assocEvento = new FpaassoceventoEntity($entityManager, $hcmParameters);
        $this->_evento = new FpaeventomEntity($entityManager, $hcmParameters);
        $this->_itemGrupo = $itensGrupo;
        $this->_hierarquiaOrg = new HierarquiaorgEntity($entityManager, $hcmParameters);
        $this->_estruturacao = new Estruturacao($entityManager, $hcmParameters);
        $this->_grrfMagnetico = $grrfMagnetico;
        $this->_gfipMagneticoTrab = new FpagfipmagnetEntity($entityManager, $hcmParameters);
        $this->_escalaTrabalho = new GpeescalatrabmEntity($entityManager, $hcmParameters);
        $this->_contaCorrenteParceiro = new ContcorrparcEntity($entityManager, $hcmParameters);
        $this->_valorReferencia = new FpaValorreferenciamEntity($entityManager, $hcmParameters);
        $this->_estruturam = new EstruturamEntity($entityManager, $hcmParameters);
        $this->_registro10 = new Registro10GRRF();
        $this->_registro40 = new Registro40GRRF();
        $this->_registro43 = new Registro43GRRF();
        $this->_registro44 = new Registro44GRRF();
        $this->_registro45 = new Registro45GRRF();
        $this->_formulario = $formulario;
        $this->_inicializaVariaveis($formulario, $competencia13);
    }

    private function _inicializaVariaveis($formulario, $competencia13){
        
        try {
            $this->_estruturacao->carregaHierarquias($formulario['NRHIERARQORG']); 
        } catch(\Exception $e){
            throw new \Exception('A Hierarquia selecionada não existe para o(s) vínculo(s) selecionado(s)');
        }
        try{
            $this->_estruturacao->carregaHierarquiasTomador($formulario['NRHIERARTOMADOR']);
        } catch(\Exception $e){
            throw new \Exception('A Hierarquia Tomador selecionada não existe para o(s) vínculo(s) selecionado(s)');
        }
        $this->_estruturacao->carregaTipoHierarquia($formulario['NRHIERARQORG']);
        $this->_estruturacao->carregaTipoHierarquiaTomador($formulario['NRHIERARTOMADOR']);
        
        $this->_registro10->setCodigoRecolhimento($formulario['NRRECOLHIMENTO']);
        $this->_registro10->setCodCentralizadorFGTS($formulario['IDCENTRALFGTS']);
        $this->_registro10->setCNPJCentralizador($formulario['NRINSCCENTRALIZ']);
        $this->_registro40->setCodigoRecolhimento($formulario['NRRECOLHIMENTO']);
        $this->_registro40->setDtHomologacao($formulario['DTHOMOLOGACAO']);
        $this->_registro40->setCompetencia($this->_mesCompetencia);
        $this->_registro40->setCompetencia13($competencia13);
        $this->_registro40->setFGTSMesAnterior($formulario['FGTSMESANTERIOR']);
        
        $this->_infCompl43 = $formulario['IDINFOCOMPL43'];
        $this->_infCompl44 = $formulario['IDINFOCOMPL44'];
        $this->_infCompl45 = $formulario['IDINFOCOMPL45'];
        $this->_infComplComps = array_reverse($formulario['DTMESCOMPETENCLIST']);
    }

    public function gerarGRRF($vinculo, $tpInscHierarq, $tpInscHierarqTom){

        try {
            $this->_calculaBasesGRRF($vinculo->getNrvinculom());
        } catch (\Exception $e) {
            throw $e;
        }    

        $estruturaResp = $this->entityManager->getRepository(Repositories::GPE_MOVIMENTACAO)->getMovimentacao($this->_nrOrg, FPA::getTipoEstruturaLegalOrg(),
            $vinculo->getNrvinculom());
        
        if ($estruturaResp) {
            $estrutura = $this->_estruturam->retornaEstrutura($estruturaResp[0]->getNrestruturam());
            if ($tpInscHierarq === RegistroGRRF::CNPJ) {
                $this->_nrInscricao = preg_replace(
                    self::ALFANUMERICOS,
                    '',
                    $estrutura->historico->getCdcnpjestrut()
                );
                $this->_nrTpInscricao = RegistroGRRF::CNPJ;
            } elseif ($tpInscHierarq === RegistroGRRF::CEI) {
                $this->_nrInscricao = preg_replace(
                    self::ALFANUMERICOS, '',
                    $estrutura->historico->getCdceiestrut()
                );
                $this->_nrTpInscricao = RegistroGRRF::CEI;
            }

            $this->_preparaRegistro10($estrutura, $vinculo->getNrvinculom());
            $ocupacao = $this->_ocupacao->retornaOcupacao($vinculo->historico->getNrcargo());

            if ($tpInscHierarqTom) {
                $movimentacaoGeren = $this->entityManager->getRepository(Repositories::GPE_MOVIMENTACAO)->getMovimentacao($this->_nrOrg, FPA::getTipoEstruturaGerencialOrg(),
                $vinculo->getNrvinculom());
                
                if($movimentacaoGeren) {
                    $estruturaTomador =  $this->_estruturacao->getEstruturaSuperiorTomador($movimentacaoGeren[0]);
                    if($estruturaTomador){
                        $this->_preparaRegistro40($vinculo, $ocupacao, $estruturaTomador);
                    }
                }
            }else{
                $this->_preparaRegistro40($vinculo, $ocupacao, $estruturaResp);
            }
        }
    }

    private function _calculaBasesGRRF($nrVinculo){
        $this->_dadoVinculo = array();
        $this->_dadoMesAnteriorRescisao = array();
        $this->_dadoMesRescisao = array();
        $this->_dadoAvisoPrevioIndenizado = array();
        $this->_dadoSaldoFinsRescisorios = array();
        $this->_dadoPensaoJuridica = array();
        $this->_dadoInfComplCompAnt = array();
        
        foreach ($this->_listaCalculoFolha as $nrCalculo) {

            $this->_itemCalculo->retornaItensPorVinculo($this->_nrOrg, $nrVinculo, $nrCalculo);

            $itens = $this->_itemCalculo->getItems();
            if(is_array($itens)) {
                $idSemDefinitivo = true;
                foreach($itens as $val) {
                    if($val->getNreventom() == self::NREVENTOLIQPARCIAL) {
                        MsgCalculo::getInstance()->adicionaMensagem('Vínculo '.$nrVinculo.' não possui líquido definitivo de rescisão.');
                        //break;
                    }
                    if($val->getNreventom() == self::NREVENTOLIQDEFINITIVO) {
                        $idSemDefinitivo = false;
                    }
                }
                if($idSemDefinitivo)
                    MsgCalculo::getInstance()->adicionaMensagem('Vínculo '.$nrVinculo.' não possui líquido definitivo de rescisão.');
            } else {
                MsgCalculo::getInstance()->adicionaMensagem('Não foram encontreados itens de cálculo para o vínculo nro '.$nrVinculo.'.');
            }
            $this->_mesAnteriorRescisao();
            $this->_mesRescisao();
            $this->_avisoPrevioIndenizado();
            $this->_SaldoFinsRescisorios();
            $this->_pensaoJuridica();

        }
        
        foreach ($this->_listaCalculoFolhaAnt as $mescompetencia => $listaCalculoComp) {
            foreach ($listaCalculoComp as $nrCalculoComp) {
                $this->_itemCalculo->retornaItensPorVinculo($this->_nrOrg, $nrVinculo, $nrCalculoComp);
                $this->_infComplCompAnterior($mescompetencia);
            }
        }
    }

    private function _preparaRegistro10($estrutura, $nrVinculo){

        $posicao = $this->_grrfMagnetico->procuraRegistroLista(
            $this->_nrInscricao, "0", Registro10GRRF::REGISTRO_10
        );

        if ($posicao === false) {

            $endereco = $this->_estruturacao->determinaEnderecoEstrutura($estrutura);
            $this->_registro10->setTpInscricao($this->_nrTpInscricao);
            $this->_registro10->setNrInscricao($this->_nrInscricao);
            
            $this->_registro10->geraRegistro10($estrutura, $endereco);

            $this->_grrfMagnetico->insereRegistroLista(
                $this->_nrOrg, $this->_cdOperador, $this->_mesCompetencia, Registro10GRRF::REGISTRO_10, $this->_nrTpInscricao,
                $this->_nrInscricao, $this->_registro10->getLinhaRegistro(), '0', '0', null, null, '0', null, null, null, $this->_formulario['FGTSMESANTERIOR']
            );
            $this->_registro10->setLinhaRegistro();
        }
    }

    private function _preparaRegistro40($vinculo, $ocupacao, $estrutura){
        
        if($vinculo->getNrtpdemissao()){
            $objTpDemissao = $this->_tpDemissao->procuraTipoDemissao($vinculo->getNrtpdemissao());
            $this->_codMovimentacao = $objTpDemissao->getCodfgts();
            $this->_codSaque = $objTpDemissao->getCodsaque();

        }else{
            MsgCalculo::getInstance()->adicionaMensagem('Tipo de demissao nao parametrizado para o vinculo '.$vinculo->getNrvinculom());
        }
        
        $codigomovimentacao = array('I1','I2','I3','I4','I5','L');
        if (array_search($this->_codMovimentacao, $codigomovimentacao) !== false) {

            $this->_dependentes->carregaDependenciasPorVinculo($vinculo->getNrvinculom());
            $dependentes = $this->_dependentes->getItems();

            $nrEscalaTrabalho = $vinculo->historico->getNrescalatrabm();
            $escalaTrabalho = $this->_escalaTrabalho->retornaEscala($nrEscalaTrabalho, $this->_nrOrg);
            if (is_object($escalaTrabalho) === true) {
                $vinculo->escalaSemanal = $escalaTrabalho->historico->getQthrsemesctrabh();
            } else {
                MsgCalculo::getInstance()->adicionaMensagem('Escala Semanal nao parametrizado para o vinculo '.$vinculo->getNrvinculom());
            }
            
            if (is_object($ocupacao) === true){
                $cbo = $this->_cbo->retornaCbo($ocupacao->historico->getNrcbo());
                if ($cbo) {
                    $vinculo->cbo = $cbo->getCdrais();
                } else {
                    MsgCalculo::getInstance()->adicionaMensagem('Nao parametrizado cbo para o cargo do vinculo '.$vinculo->getNrvinculom());
                }
            } else {
                MsgCalculo::getInstance()->adicionaMensagem('Nao parametrizado cargo para o vinculo '.$vinculo->getNrvinculom());
                $vinculo->cbo = '';
            }

            $situacaoFuncional = $this->_sitFuncional->retornaSituacaoFuncional(
                $this->_nrOrgPadrao, $this->_nrOrg, $vinculo->historico->getNrsitufuncm()
            );

            if (is_object($situacaoFuncional) === true) {
                $nrCategoriaTrabalhador = $situacaoFuncional->historico->getCodcategtrab();
                if (empty($nrCategoriaTrabalhador) === true) {
                    $vinculoEmpreg = $this->_vinculoEmpreg->retornaVinculoEmpregaticio($vinculo->getNrvinculoempreg());
                    if (is_object($vinculoEmpreg) === true) {
                        $nrCategoriaTrabalhador = $vinculoEmpreg->getCodcategtrab();
                    }
                }
            } else {
                MsgCalculo::getInstance()->adicionaMensagem('Situação Funcional não encontrada para o vinculo '.$vinculo->getNrvinculom(), MsgCalculo::ERRO, true);
            }

            $pessoa = $this->_pessoa->retornaPessoaAtual($vinculo->getNrpessoa());
            
            if (is_null($pessoa->historico->getDtnascpessoa())) {
                MsgCalculo::getInstance()->adicionaMensagem('Pessoa '.$pessoa->getNrpessoa().' '.$pessoa->historico->getNmpessoa().'sem data de nascimento informada.', MsgCalculo::ERRO, true);
            }
            
            $contaCorrentePessoa = $this->_contaCorrenteParceiro->retornaContaCorrentePorParceiro($pessoa->getNrparcnegocio());
            if($contaCorrentePessoa){
                $this->_registro40->setBanco($contaCorrentePessoa->getCdbanco());
                $this->_registro40->setAgencia($contaCorrentePessoa->getCdagencia());
                $this->_registro40->setContaCorrente($contaCorrentePessoa->getCdcontcorr());
            }else{
                $this->_registro40->setBanco(null);
                $this->_registro40->setAgencia(null);
                $this->_registro40->setContaCorrente(null);
                MsgCalculo::getInstance()->adicionaMensagem('Parceiro de negocio '.$pessoa->getNrparcnegocio().' foi gerado sem conta corrente.');
            }

            $this->_registro40->setNrInscricao($this->_nrInscricao);
            $this->_registro40->setTpInscricao($this->_nrTpInscricao);
            $this->_registro40->setNrInscricaoTomador($this->_nrInscricaoTomador);
            $this->_registro40->setTpInscricaoTomador($this->_nrTpInscricaoTomador);
            $this->_registro40->setCodMovimentacao($this->_codMovimentacao);
            $this->_registro40->setCodSaque($this->_codSaque);
            $this->_registro40->setEscalaSemanal($vinculo->escalaSemanal);
            $this->_registro40->setNrCategoria($nrCategoriaTrabalhador);
            $this->_registro40->setTipoDependente($dependentes);
            
            $this->_registro40->setIdAvisoTrabAsInden($this->_valorReferencia->getValorReferencia(FpaValorreferenciamEntity::AVISO_TRAB_INDEN_GRRF));

            $this->_registro40->geraRegistro40($pessoa, $vinculo, $this->_dadoAvisoPrevioIndenizado, $this->_dadoMesAnteriorRescisao, $this->_dadoMesRescisao, $this->_dadoPensaoJuridica, $this->_dadoSaldoFinsRescisorios);

            $nrPisPasepCi = $pessoa->historico->getNrpispaseppes();

            $this->_grrfMagnetico->persisteRegistro($this->_nrOrg, $this->_cdOperador, $this->_mesCompetencia, Registro40GRRF::REGISTRO_40,
                $this->_nrTpInscricao, $this->_nrInscricao, $this->_registro40->getLinhaRegistro(),
                $vinculo->getNrvinculom(), $vinculo->getDtadmissaovinc(), $nrPisPasepCi, $nrCategoriaTrabalhador,
                $this->_formulario['FGTSMESANTERIOR']);

            $this->_registro40->setLinhaRegistro();
            
            //Registros de Informações Complementares
            $this->_preparaRegistrosInfCompl($pessoa, $vinculo, $nrCategoriaTrabalhador);

        } else {
            MsgCalculo::getInstance()->adicionaMensagem("Cód. FGTS do Tipo de Demissão do vínculo ".$vinculo->getNrvinculom()." não entra para GRRF.");            
        }

    }

    private function _preparaRegistrosInfCompl($pessoa, $vinculo, $nrCategoriaTrabalhador){
        
        if ($this->_codMovimentacao != 'I3' && $nrCategoriaTrabalhador != '05') {
            
            if ($this->_infCompl43 == 'S') {
                $this->_registro43->setNrInscricao($this->_nrInscricao);
                $this->_registro43->setTpInscricao($this->_nrTpInscricao);
                $this->_registro43->setNrCategoria($nrCategoriaTrabalhador);
                
                $this->_registro43->setCompentencias($this->_dadoInfComplCompAnt);
    
                $this->_registro43->geraRegistro43($pessoa, $vinculo);
    
                $this->_grrfMagnetico->persisteRegistro($this->_nrOrg, $this->_cdOperador, $this->_mesCompetencia, Registro43GRRF::REGISTRO_43,
                    $this->_nrTpInscricao, $this->_nrInscricao, $this->_registro43->getLinhaRegistro(),
                    $vinculo->getNrvinculom(), $vinculo->getDtadmissaovinc(), $pessoa->historico->getNrpispaseppes(), $nrCategoriaTrabalhador,
                    $this->_formulario['FGTSMESANTERIOR']);
    
                $this->_registro43->setLinhaRegistro();
            }
            
            if ($this->_infCompl44 == 'S') {
                $this->_registro44->setNrInscricao($this->_nrInscricao);
                $this->_registro44->setTpInscricao($this->_nrTpInscricao);
                $this->_registro44->setNrCategoria($nrCategoriaTrabalhador);
                
                $this->_registro44->setCompentencias($this->_dadoInfComplCompAnt);
    
                $this->_registro44->geraRegistro44($pessoa, $vinculo);
    
                $this->_grrfMagnetico->persisteRegistro($this->_nrOrg, $this->_cdOperador, $this->_mesCompetencia, Registro44GRRF::REGISTRO_44,
                    $this->_nrTpInscricao, $this->_nrInscricao, $this->_registro44->getLinhaRegistro(),
                    $vinculo->getNrvinculom(), $vinculo->getDtadmissaovinc(), $pessoa->historico->getNrpispaseppes(), $nrCategoriaTrabalhador,
                    $this->_formulario['FGTSMESANTERIOR']);
    
                $this->_registro44->setLinhaRegistro();
            }
            
            if ($this->_infCompl45 == 'S') {
                $this->_registro45->setNrInscricao($this->_nrInscricao);
                $this->_registro45->setTpInscricao($this->_nrTpInscricao);
                $this->_registro45->setNrCategoria($nrCategoriaTrabalhador);
                
                $this->_registro45->setCompentencias($this->_dadoInfComplCompAnt);
    
                $this->_registro45->geraRegistro45($pessoa, $vinculo);
    
                $this->_grrfMagnetico->persisteRegistro($this->_nrOrg, $this->_cdOperador, $this->_mesCompetencia, Registro45GRRF::REGISTRO_45,
                    $this->_nrTpInscricao, $this->_nrInscricao, $this->_registro45->getLinhaRegistro(),
                    $vinculo->getNrvinculom(), $vinculo->getDtadmissaovinc(), $pessoa->historico->getNrpispaseppes(), $nrCategoriaTrabalhador,
                    $this->_formulario['FGTSMESANTERIOR']);
    
                $this->_registro45->setLinhaRegistro();
            }
        } else {
            MsgCalculo::getInstance()->adicionaMensagem("Cód. FGTS do Tipo de Demissão do vínculo ".$vinculo->getNrvinculom()." não entra para GRRF.");            
        }

    }

    private function _calculaValorItemGrupo($referencia){
        $valor = $this->_itemGrupo->calculaValorItemGrupoIncidencia($this->_mesCompetencia, $this->_itemCalculo,
            $this->_evento, $referencia);
        return $valor;
    }

    private function _mesAnteriorRescisao(){
        $valor = $this->_calculaValorItemGrupo(RegistroGRRF::GRRF_MES_ANTERIOR_RESCISAO);
        $this->_consolidaDadosMesAnteriorRescisao(RegistroGRRF::GRRF_MES_ANTERIOR_RESCISAO, $valor);
    }

    private function _mesRescisao(){
        $valor = $this->_calculaValorItemGrupo(RegistroGRRF::GRRF_MES_RESCISAO);
        $this->_consolidaDadosMesRescisao(RegistroGRRF::GRRF_MES_RESCISAO, $valor);
    }

    private function _avisoPrevioIndenizado(){
        $valor = $this->_calculaValorItemGrupo(RegistroGRRF::GRRF_AVISO_PREVIO_INDENIZADO);
        $this->_consolidaDadosAvisoPrevioIndenizado(RegistroGRRF::GRRF_AVISO_PREVIO_INDENIZADO, $valor);
        $valor = $this->_calculaValorItemGrupo(RegistroGRRF::GRRF_COMPL_AVISO_PREVIO_INDENIZADO);
        $this->_consolidaDadosAvisoPrevioIndenizado(RegistroGRRF::GRRF_COMPL_AVISO_PREVIO_INDENIZADO, $valor);
    }

    private function _SaldoFinsRescisorios(){
        $valor = $this->_calculaValorItemGrupo(RegistroGRRF::GRRF_SALDO_FINS_RESCISORIOS);
        $this->_consolidaDadosSaldoFinsRescisorios(RegistroGRRF::GRRF_SALDO_FINS_RESCISORIOS, $valor);
    }

    private function _pensaoJuridica(){
        $valor = $this->_calculaValorItemGrupo(RegistroGRRF::GRRF_PENSAO_JURIDICA);
        $this->_consolidaDadosPensaoJuridica(RegistroGRRF::GRRF_PENSAO_JURIDICA, $valor);
    }

    private function _infComplCompAnterior($mescompetencia){
        $valor = $this->_calculaValorItemGrupo(RegistroGRRF::GRRF_COMP_INF_COMPL);
        $this->_consolidaDadosCompInfCompl(RegistroGRRF::GRRF_COMP_INF_COMPL, $mescompetencia, $valor);
    }

    private function _consolidaDadosAvisoPrevioIndenizado($referencia, $valor) {
        $this->_dadoAvisoPrevioIndenizado[$referencia] += $valor;
    }

    private function _consolidaDadosMesAnteriorRescisao($referencia, $valor) {
        $this->_dadoMesAnteriorRescisao[$referencia] += $valor;
    }

    private function _consolidaDadosMesRescisao($referencia, $valor) {
        $this->_dadoMesRescisao[$referencia] += $valor;
    }

    private function _consolidaDadosSaldoFinsRescisorios($referencia, $valor) {
        $this->_dadoSaldoFinsRescisorios[$referencia] += $valor;
    }

    private function _consolidaDadosPensaoJuridica($referencia, $valor) {
        $this->_dadoPensaoJuridica[$referencia] += $valor;
    }

    private function _consolidaDadosCompInfCompl($referencia, $mescompetencia, $valor) {
        $this->_dadoInfComplCompAnt[$referencia][$mescompetencia] += $valor;
    }

    public function retroceder(){
        try{
            $this->_grrfMagnetico->retroceder($this->_nrOrg, $this->_cdOperador, $this->_mesCompetencia);
            MsgCalculo::getInstance()->adicionaMensagem("Retrocedido");
        }catch (Exception $e){
            MsgCalculo::getInstance()->adicionaMensagem($e->getMessage());
        }
    }
}