<?php

namespace HCM\FPA\Services;

use Doctrine\ORM\EntityManager;

class RateioLancamentoService {
    
    protected $entityManager;
    
    public function __construct(EntityManager $entityManager) {
        $this->entityManager = $entityManager;
    }

    public function buscaRateioExistente($nrorg, $inicioVigencia, $nrestruturam, $nreventom) {
        $sql = $this->entityManager->getConnection()->prepare(
            "SELECT RAT.* 
            FROM FPA_RATEIO RAT
            WHERE (RAT.NRORG = :P_NRORG 
            AND RAT.DTINIVIGENCIA = :P_DTINIVIGENCIA
            AND RAT.NRESTRUTURAM = :P_NRESTRUTURAM
            AND RAT.NREVENTOM = :P_NREVENTOM
            )"
        );
        
        $sql->execute(
            array(
                'P_NRORG' => $nrorg,
                'P_DTINIVIGENCIA' => $inicioVigencia,
                'P_NRESTRUTURAM' => $nrestruturam,
                'P_NREVENTOM' => $nreventom
            )
        );
        
        return $sql->fetch();
    }

    public function buscaRateioExistenteOcupacao($nrorg, $nrestruturam, $nrocupacaom, $nrrateio) {
        $sql = $this->entityManager->getConnection()->prepare(
            "SELECT RAT.*
            FROM FPA_RATEIOOCUP RAT 
            WHERE (RAT.NRORG = :P_NRORG
            AND RAT.NRRATEIO = :P_NRRATEIO
            AND  RAT.NRESTRUTURAM = :P_NRESTRUTURAM
            AND RAT.NROCUPACAOM = :P_NROCUPACAOM)"
        );
        $sql->execute(
            array(
                'P_NRORG' => $nrorg,
                'P_NRESTRUTURAM' => $nrestruturam,
                'P_NROCUPACAOM' => $nrocupacaom,
                'P_NRRATEIO' => $nrrateio
            )
        );
        
        return $sql->fetch();
    }

    public function buscaCalculoRateioExistente($nrorg, $nrrateio, $nrvinculom, $dtdialanc) {
        $sql = $this->entityManager->getConnection()->prepare(
            "SELECT RAT.*
            FROM FPA_RATEIOVAL RAT 
            WHERE (RAT.NRORG = :P_NRORG
            AND RAT.NRRATEIO = :P_NRRATEIO
            AND  RAT.NRVINCULOM = :P_NRVINCULOM
            AND RAT.DTDIALANC = :P_DTDIALANC)"
        );
        $sql->execute(
            array(
                'P_NRORG' => $nrorg,
                'P_DTDIALANC' => $dtdialanc,
                'P_NRVINCULOM' => $nrvinculom,
                'P_NRRATEIO' => $nrrateio
            )
        );
        
        return $sql->fetch();
    }

    public function buscaRateioExistenteEstrutura($nrorg, $nrestruturam, $nrrateio) {
        $sql = $this->entityManager->getConnection()->prepare(
            "SELECT RAT.*
            FROM FPA_RATEIOESTRUT RAT 
            WHERE (RAT.NRORG = :P_NRORG
            AND RAT.NRRATEIO = :P_NRRATEIO
            AND    RAT.NRESTRUTURAM = :P_NRESTRUTURAM)"
        );
        $sql->execute(
            array(
                'P_NRORG' => $nrorg,
                'P_NRESTRUTURAM' => $nrestruturam,
                'P_NRRATEIO' => $nrrateio
            )
        );
        
        return $sql->fetch();
    }

    public function buscaRateioExistenteLancamento($nrorg, $nrrateio, $dtlancamento) {
        $sql = $this->entityManager->getConnection()->prepare(
        "SELECT * FROM FPA_RATEIOLANC RAT WHERE (NRORG = :P_NRORG AND RAT.NRRATEIO = :P_NRRATEIO AND RAT.DTLANCAMENTO = :P_DTLANCAMENTO)"
        );
        
        $sql->execute(
            array(
                'P_NRORG' => $nrorg,
                'P_NRRATEIO' => $nrrateio,
                'P_DTLANCAMENTO' => $dtlancamento
            )
        );
        
        return $sql->fetch();
    }
     
    public function existeItensRateio($nrorg, $nrrateio) {
        $existeItemEstrutura =  self::selectExisteItensRateio($nrorg, $nrrateio, 'FPA_RATEIOESTRUT');
        $existeItemLancamento =  self::selectExisteItensRateio($nrorg, $nrrateio, 'FPA_RATEIOLANC');
        $existeItemOcupacao = self::selectExisteItensRateio($nrorg, $nrrateio, 'FPA_RATEIOOCUP' );
        if($existeItemEstrutura || $existeItemLancamento || $existeItemOcupacao) {
            return true;
        }

        return false;
    }
    

    private function selectExisteItensRateio($nrorg, $nrrateio, $tabela) {
        $sql = $this->entityManager->getConnection()->prepare(
            "SELECT * FROM $tabela WHERE (NRORG = :P_NRORG AND NRRATEIO = :P_NRRATEIO)"
            );
            
            $sql->execute(
                array(
                    'P_NRORG' => $nrorg,
                    'P_NRRATEIO' => $nrrateio
                )
            );
          
            return $sql->fetch();
    }

    public function calculaPorcentagem($nrorg, $nrrateio, $tabela)
     {
        if($tabela == 'FPA_RATEIOESTRUT') {
            $porcentagens = self::selectPorcentagemEstrutura($nrorg, $nrrateio);
            foreach($porcentagens as $porcentagem) {
                self::setPorcentagemEstrutura($nrorg, $nrrateio, $porcentagem['NRRATEIOESTRUT'],$porcentagem['PERCENT']);
            }
        } 

        if($tabela == 'FPA_RATEIOOCUP') {
            $porcentagens = self::selectPorcentagemOcupacao($nrorg, $nrrateio);
            foreach($porcentagens as $porcentagem) {
                self::setPorcentagemOcupacao($nrorg, $nrrateio, $porcentagem['NRRATEIOOCUP'],$porcentagem['PERCENT']);
            }
        }
         
    }

    private function selectPorcentagemEstrutura($nrorg, $nrrateio)
    {
        $sql = $this->entityManager->getConnection()->prepare(
            "SELECT RAT.NRRATEIOESTRUT, ( RAT.NRPESO / SUM(RAT.NRPESO) OVER(PARTITION BY RAT.NRORG, RAT.NRRATEIO)) * 100 PERCENT FROM FPA_RATEIOESTRUT RAT WHERE RAT.NRORG = :P_NRORG AND RAT.NRRATEIO = :P_NRRATEIO"
            );
            
            $sql->execute(
                array(
                    'P_NRORG' => $nrorg,
                    'P_NRRATEIO' => $nrrateio
                )
            );
            
            return $sql->fetchAll();
    }

    private function selectPorcentagemOcupacao($nrorg, $nrrateio)
    {
        $sql = $this->entityManager->getConnection()->prepare(
            "SELECT RAT.NRRATEIOOCUP, ( RAT.NRPESO / SUM(RAT.NRPESO) OVER(PARTITION BY RAT.NRORG, RAT.NRRATEIO, RAT.NRESTRUTURAM)) * 100 PERCENT FROM FPA_RATEIOOCUP RAT WHERE RAT.NRORG = :P_NRORG AND RAT.NRRATEIO = :P_NRRATEIO"
            );
            
            $sql->execute(
                array(
                    'P_NRORG' => $nrorg,
                    'P_NRRATEIO' => $nrrateio
                )
            );
            
            return $sql->fetchAll();
    }

    private function setPorcentagemEstrutura($nrorg, $nrrateio, $nrrateioestrut, $vrporcentagem)
    {
        $sql = $this->entityManager->getConnection()->prepare("
            UPDATE FPA_RATEIOESTRUT 
            SET VRPORCENTAGEM     = :P_VRPORCENTAGEM 
            WHERE NRRATEIO     = :P_NRRATEIO
                AND NRRATEIOESTRUT  = :P_NRRATEIOESTRUT
                AND NRORG           = :P_NRORG");

            $sql->execute(
                array(
                    "P_VRPORCENTAGEM"  => $vrporcentagem,
                    "P_NRRATEIO"     => $nrrateio,
                    "P_NRRATEIOESTRUT"  => $nrrateioestrut,
                    "P_NRORG"           => $nrorg
                )
            );
    }

    private function setPorcentagemOcupacao($nrorg, $nrrateio, $nrrateioocup, $vrporcentagem)
    {
        $sql = $this->entityManager->getConnection()->prepare("
            UPDATE FPA_RATEIOOCUP 
            SET VRPORCENTAGEM     = :P_VRPORCENTAGEM 
            WHERE NRRATEIO     = :P_NRRATEIO
                AND NRRATEIOOCUP  = :P_NRRATEIOOCUP
                AND NRORG           = :P_NRORG");

            $sql->execute(
                array(
                    "P_VRPORCENTAGEM"  => $vrporcentagem,
                    "P_NRRATEIO"     => $nrrateio,
                    "P_NRRATEIOOCUP"  => $nrrateioocup,
                    "P_NRORG"           => $nrorg
                )
            );
    }

    
}