<?php

namespace HCM\Geral\Entity;

use HCM\Util\TkEntity;
use HCM\Util\Repositories;
use HCM\Util\DateUtil;

class EstruturamEntity extends TkEntity
{
    private $_nrOrg;
    private $_mesCompetencia;

    private $_nrEstruturaAnterior;
    private $_estrutura;
    public $historico;

    public function __construct($entityManager, $hcmParameters) {
        $this->_nrOrg = $hcmParameters->getNrorgTrabalho();
        $this->_mesCompetencia = $hcmParameters->getDtCompetencia();
        
        $this->entityManager = $entityManager;
        $this->hcmParameters = $hcmParameters;
        parent::__construct(null, $entityManager);
    }

    public function getEstrutura(){
        return $this->_estrutura;
    }

    public function insereEstrutura($nrorg, $nrtipoestrutura, $cdintestrutura, $dtinivigencia) {
        $this->setNrorg($nrorg);
        $this->setNrtipoestrutura($nrtipoestrutura);
        $this->setCdintestrutura($cdintestrutura);
        $this->setDtinivigencia($dtinivigencia);
    }

    public function carregaEstrutura($nrEstrutura){

        try {
            if ($this->_nrEstruturaAnterior !== $nrEstrutura) {
                $posicao = $this->_procuraEstrutura($nrEstrutura);
                if ($posicao === -1) {

                    $this->_estrutura = $this->retornaEstrutura($nrEstrutura);
                    $this->addItem($this->_estrutura);
                    $this->_nrEstruturaAnterior = $nrEstrutura;
                } else {
                    $this->_estrutura = $this->getItem($posicao);
                }
            }
        } catch (Exception $e){
            throw  $e;
        }

    }

    private function _procuraEstrutura($nrEstrutura) {

        $itens = $this->getItems();

        foreach ($itens as $key=>$item) {
            if ($item) {
                if (($item->getNrorg() === $this->_nrOrg) && ($item->getNrestruturam() === $nrEstrutura)) {
                    return $key;
                }
            }
        }

        return -1;
    }

    public function retornaEstrutura($nrEstrutura){
       $this->_ult_dia_mes = DateUtil::getUltimoDiaDoMes($this->_mesCompetencia);
        $estrutura = $this->entityManager->getRepository(Repositories::ESTRUTURAM)->retornaEstrutura($this->_nrOrg, $nrEstrutura, $this->_ult_dia_mes);
        
        if (is_object($estrutura) === true) {
            
            $historico = $this->entityManager->getRepository(Repositories::ESTRUTURAH)->retornaEstruturaHistorico($this->_nrOrg, $nrEstrutura, $this->_mesCompetencia);
            $estrutura->historico = $historico;
            return $estrutura;
        } else {
            return null;
        }
    }

    public function novoMestre($nrestruturam, $nrtipoestrutura, $nrorgTrab){
        try{
            $this->setNrestruturam($nrestruturam);
            $this->setNrtipoestrutura($nrtipoestrutura);
            $this->setDtinivigencia($now);
            $this->setNrorg($nrorgTrab);
            $this->persist();
            return true;
        }
        catch (Exception $e){
            return false;
        }

    }

    public function destroiMestre($nrestruturam, $nrorgTrab){
        try{
            $estruturam = Estruturam::retornaEstruturaPelaChave($nrestruturam, $nrorgTrab);
            if (is_object($estruturam) === true) {
                $this->addItem($estruturam, $nrestruturam);
                $this->removeItem($estruturam);
                $this->persist();
                return true;
            } else {
                return false;
            }
        }
        catch (Exception $e){
            return false;
        }

    }


    public function criaEstruturaMestre($nrOrg, $dtIniVigencia, $tipoEstrutura, $cdInterno){

        $estrutura = new self();

        $estrutura->setNrorg($nrOrg);
        $estrutura->setDtinivigencia($dtIniVigencia);
        $estrutura->setNrtipoestrutura($tipoEstrutura);
        $estrutura->setCdintestrutura($cdInterno);

        $estrutura->persist();

        $this->setNrestruturam($estrutura->getNrestruturam());

        unset($estrutura);

    }


    public function retornaEstruturaPorCdIntegracao($cdIntegracao){
        $resultado = Estruturam::retornaEstruturaPorCdIntegracao($this->_nrOrg, $cdIntegracao);
        return $resultado;
    }

    public static function retornaEstruturaPorCdintestrutura($nrorg, $cdintestrutura) {
        $estruturam = self::getEntityByWhere( 'NRORG = ? AND CDINTESTRUTURA = ?', array($nrorg, $cdintestrutura));
        return $estruturam;
    }

    public function retornaEstruturaPorNrestruturam($nrorg, $nrestruturam) {
      
        $estruturam = $this->entityManager->getRepository(Repositories::ESTRUTURAM)->findOneBy(array('nrorg' => $nrorg,
        'nrestruturam' => $nrestruturam));
        
        $this->historico = null;
        $estruturam->historico = $this->getHistorico($nrorg, $nrestruturam);
        
        return $estruturam;
    }
    
    public static function retornaEstruturaPorCdintestruturaETipo($nrorg, $cdintestrutura, $nrtipoestrutura) {
        $estruturam = self::getEntityByWhere( 'NRORG = ? AND CDINTESTRUTURA = ? AND NRTIPOESTRUTURA = ?', array($nrorg, $cdintestrutura, $nrtipoestrutura));
        return $estruturam;
    }

    public function getHistorico($nrorg,$nrestruturam) {
        if (!$this->historico) {
            $this->historico = $this->entityManager->getRepository(Repositories::ESTRUTURAH)->findOneBy(array('nrorg' => $nrorg,
            'nrestruturam' => $nrestruturam),array('dtmescompetenc' => 'DESC'));
        }
        return $this->historico;
    }

    public function getEstruturaSuperior($nrTipoHierarquiaTomador = null, $estrutura = null) {
        if (!empty($nrTipoHierarquiaTomador)) {
            $hierarqorg = $this->entityManager->getRepository(Repositories::HIERARQUIAORG)->findOneBy(array('nrorg' => $this->_nrOrg,
            'nrestruturamrel' => $estrutura->getNrestruturam(), 'nrtipohierarqorg' => $nrTipoHierarquiaTomador));
        } else {
            $hierarqorg = $this->entityManager->getRepository(Repositories::HIERARQUIAORG)->findOneBy(array('nrorg' => $this->_nrOrg,
            'nrestruturamrel' => $estrutura->getNrestruturam()));
        }

        if (!$hierarqorg) {
            return null;
        }
        
        $estruturaSuperior = $this->entityManager->getRepository(Repositories::ESTRUTURAM)->findOneBy(array('nrorg' => $this->_nrOrg,
        'nrestruturam' => $hierarqorg->getNrestruturamsup()));
        
        return $estruturaSuperior;
    }

    public function getEstruturasRelacionadas($nrTipoHierarquia = null) {
        if (!empty($nrTipoHierarquia)) {
            $hierarqorg = HierarquiaorgEntity::getEntitiesByWhere(
                'NRORG = ? AND NRESTRUTURAMSUP = ? AND NRTIPOHIERARQORG = ?', array($this->getNrorg(), $this->getNrestruturam(), $nrTipoHierarquia)
            );
        } else {
            $hierarqorg = HierarquiaorgEntity::getEntitiesByWhere(
                'NRORG = ? AND NRESTRUTURAMSUP = ?', array($this->getNrorg(), $this->getNrestruturam())
            );
        }

        if (empty($hierarqorg)) {
            return null;
        }

        $estruturasRelacionadas = array();

        foreach($hierarqorg as $estrutSup){
            $estruturaRelacionada = self::getEntityByWhere(
                'NRORG = ? AND NRESTRUTURAM = ?',
                array($this->getNrorg(), $estrutSup->getNrestruturamrel())
            );
            if (is_object($estruturaRelacionada)){
                $estruturasRelacionadas[] = $estruturaRelacionada;
            }
        }

        return $estruturasRelacionadas;
    }
}