<?php
namespace Fiscal\Util;
use \Estoque\Logic\AtualizacaoEstoque\AtualizadorDeEstoque;
/**
 * Created by JetBrains PhpStorm.
 * User: gustavoavila
 * Date: 17/12/12
 * Time: 10:40
 * To change this template use File | Settings | File Templates.
 */
class Util
{

    /**
     * Executa a replicacao de documentos, caso este esteja parametrizado para tal.
     *
     * @static
     * @param \Fiscal\Entities\Notafiscal $notafiscal
     * @throws \Exception
     */
    public static function replicacaoAutomatica(\Fiscal\Entities\Notafiscal $notafiscal)
    {
        $replicacaoAutomatica = new \Fiscal\Logic\NotaFiscal\ReplicacaoAutomatica($notafiscal);
        if ($replicacaoAutomatica->permiteReplicacaoAutomatica()) {
            try {
                $replicacaoAutomatica->geraReplicacaoAutomatica();
            } catch (\Exception $e) {
                throw $e;
            }
        }
    }

    /**
     * Geracao de rateio anterior.
     *
     * @param \Fiscal\Entities\Notafiscal $notaFiscal
     * @return array
     */
    public static function geraRateioAnterior(\Fiscal\Entities\Notafiscal $notaFiscal)
    {
        $rateio = new \Fiscal\Logic\NotaFiscal\RateioNotaFiscal($notaFiscal);
        return $rateio->geraRateioAnterior();
    }

    /**
     * Geracao de rateio contabil.
     *
     * @static
     * @param \Fiscal\Entities\Notafiscal $notaFiscal
     * @throws \Exception
     */
    public static function geraRateioPadrao(\Fiscal\Entities\Notafiscal $notaFiscal)
    {
        try {
            $geradorDeRateio = new \Fiscal\Logic\NotaFiscal\RateioNotaFiscal($notaFiscal);
            $geradorDeRateio->excluirRateio();
            $geradorDeRateio->geraRateioContabil();
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * @param \Fiscal\Entities\Notafiscal $notafiscal
     * @return array|bool
     * @throws \Exception
     */
    public static function geraVencimentos(\Fiscal\Entities\Notafiscal $notafiscal)
    {
        try {
            \Financeiro\Entities\Contareceber::getRepository()->deletaDadosNotaFiscal($notafiscal->getCdfilial(),
                                                                                      $notafiscal->getNrlanctonf(),
                                                                                      $notafiscal->getNrorg());
            return \Fiscal\Logic\NotaFiscal\ProceduresNF::callProcedureGeraVencimento($notafiscal);
        } catch(\Exception $e) {
            throw $e;
        }
    }

    /**
     * @param $numNFE string Número da NF-e [09]
     * @param $tpEmis string forma de emissão da NF-e [01] 1 – Normal – emissão normal; 2 – Contingência FS; 3 – Contingência SCAN; 4 – Contingência DPEC; 5 – Contingência FS-DA
     * @param $codUF  string Código da UF [02]
     * @param $aamm   string AAMM da emissão [4]
     * @param $cnpj   string CNPJ do Emitente [14]
     * @param $modelo string Modelo [02]
     * @param $serie  string Série [03]
     * @return string
     */
    public static function geraChaveAcessoNFE($numNFE, $tpEmis, $codUF, $aamm, $cnpj, $modelo, $serie)
    {
        //ajusta comprimento do numero
        $num = str_pad($numNFE, 9, '0',STR_PAD_LEFT);
        //calcula codigo numérico aleatório
        $cn = self::geraCN(8); //Código Numérico [08]

        //monta a chave sem o digito verificador
        $chave = "$codUF$aamm$cnpj$modelo$serie$num$tpEmis$cn";
        //calcula o digito verificador
        $dv = self::calculaDV($chave); //DV [01]

        $chave .= $dv;

        return $chave;
    }

    public static function geraCN($length=8){
        $numero = '';
        for ($x=0;$x<$length;$x++){
            $numero .= rand(0,9);
        }
        return $numero;
    }


    public static function calculaDV($chave43) {
        $soma_ponderada = 0;
        $multiplicadores = array(2,3,4,5,6,7,8,9);
        $i = 42;
        while ($i >= 0) {
            for ($m=0; $m<count($multiplicadores) && $i>=0; $m++) {
                $soma_ponderada+= $chave43[$i] * $multiplicadores[$m];
                $i--;
            }
        }
        $resto = $soma_ponderada % 11;
        if ($resto == '0' || $resto == '1') {
            return 0;
        } else {
            return (11 - $resto);
        }
    }

    /**
     * Excecuta a procedure de CUSTO_LIQUIDO.
     *
     * @static
     * @param \Fiscal\Entities\Notafiscal $notaFiscal
     */
    public static function calcularCustoLiquido(\Fiscal\Entities\Notafiscal $notaFiscal)
    {
        $lanctoestoq = \Estoque\Entities\Lanctoestoq::findOneByCdfilialAndNrlanctonfAndNrorg(
            $notaFiscal->getFilial()->getCdfilial(),
            $notaFiscal->getNrlanctonf(),
            \Geral\Util\Ambiente::getNrorg()
        );

        \Estoque\Logic\AtualizacaoEstoque\CalculaCustoLiquido::CalcularCustoLiquido(
            $lanctoestoq->getCdfilial(),
            $lanctoestoq->getNrlancestq(),
            $notaFiscal->getVrprodutos()
        );
    }

    /**
     * Executa a atualizacao de estoque e a verificacao do estoque, se este foi devidamento atualizado.
     *
     * @static
     * @param \Fiscal\Entities\Notafiscal $notaFiscal
     */
    public static function atualizaEstoque(\Fiscal\Entities\Notafiscal $notaFiscal)
    {
        /** @var $notasFiscais array Acumula todas as notas, a nota fiscal de origem e as replicas. */
        $notasFiscais   = \Fiscal\Entities\Notafiscal::findByIddocreplicAndNrorg (
            $notaFiscal->getFilial()->getCdfilial().$notaFiscal->getNrlanctonf(),
            \Geral\Util\Ambiente::getNrorg()
        );
        $notasFiscais[] = $notaFiscal;

        foreach ($notasFiscais as $NF) {
        self::executaAtualizacaoEstoque($NF);
    }
//        /** @var $lanctoestoq \Estoque\Entities\Lanctoestoq */
//        $lanctoestoq = \Estoque\Entities\Lanctoestoq::findOneBycdfilialAndnrlanctonfAndNrorg(
//            $notaFiscal->getFilial()->getCdfilial(), $notaFiscal->getNrlanctonf(), \Geral\Util\Ambiente::getNrorg()
//        );
//
//        if ($lanctoestoq !== null && AtualizadorDeEstoque::autorizaAtualzacaoDeEstoque($lanctoestoq)) {
//            self::atualizarEstoqueReplicas($notaFiscal);
//        }
    }

    /**
     *
     *
     * @static
     * @param \Fiscal\Entities\Notafiscal $notaFiscal
     * @throws \Exception
     */
    private static function executaAtualizacaoEstoque(\Fiscal\Entities\Notafiscal $notaFiscal)
    {
        /** @var $filial \Geral\Entities\Filial */
        $filial = null;
        if ($notaFiscal->getEntrsaidop()->getIdentrsaidop() == \Geral\Entities\Tipooperac::OPERACAO_ENTRADA) {
            $filial = $notaFiscal->getFilirece();
        } else {
            $filial = $notaFiscal->getFiliemit();
        }

        try {
            /** @var $lanctoestoq \Estoque\Entities\Lanctoestoq */
            $lanctoestoq = \Estoque\Entities\Lanctoestoq::findOneBycdfilialAndnrlanctonfAndNrorg(
                $filial->getCdfilial(), $notaFiscal->getNrlanctonf(), \Geral\Util\Ambiente::getNrorg()
            );

            if ($lanctoestoq !== null && AtualizadorDeEstoque::autorizaAtualizacaoDeEstoque($lanctoestoq)) {
                self::calcularCustoLiquido($notaFiscal);
                AtualizadorDeEstoque::atualizarEstoque($lanctoestoq);
                AtualizadorDeEstoque::verificarLancamentoEstoqueDaNota($notaFiscal);
                AtualizadorDeEstoque::atualizaOperadorLancamentoEstoque($lanctoestoq);
            }
        } catch (\Exception $e) {
            throw $e;
        }
    }

}
